package org.processmining.lpengines.implementations;

import java.util.HashMap;
import java.util.Map;

import lpsolve.LpSolve;
import lpsolve.LpSolveException;

import org.processmining.framework.util.Pair;

public class LPEngineLpSolveDisposableImpl extends LPEngineLpSolveImpl {

	public LPEngineLpSolveDisposableImpl() {
		super();
		try {
			this.engine = LpSolve.makeLp(0, 0);
			setup();
		} catch (LpSolveException e) {
			e.printStackTrace();
		}
	}

	public LPEngineLpSolveDisposableImpl(int rows, int columns) {
		super(rows, columns);
		try {
			this.engine = LpSolve.makeLp(rows, columns);
			setup();
		} catch (LpSolveException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void setup() {
		if (numVariables() > 0 || numConstraints() > 0) {
			this.engine.setAddRowmode(true);
		}
		this.engine.setScaling(LpSolve.SCALE_GEOMETRIC | LpSolve.SCALE_EQUILIBRATE);// | LpSolve.SCALE_INTEGERS);
		this.engine.setScalelimit(5);
		this.engine.setPivoting(LpSolve.PRICER_DEVEX | LpSolve.PRICE_ADAPTIVE);
		this.engine.setMaxpivot(250);
		this.engine.setBbFloorfirst(LpSolve.BRANCH_AUTOMATIC);
		this.engine.setBbRule(LpSolve.NODE_PSEUDONONINTSELECT | LpSolve.NODE_GREEDYMODE | LpSolve.NODE_DYNAMICMODE
				| LpSolve.NODE_RCOSTFIXING);
		this.engine.setBbDepthlimit(-50);
		this.engine.setAntiDegen(LpSolve.ANTIDEGEN_FIXEDVARS | LpSolve.ANTIDEGEN_STALLING);
		this.engine.setImprove(LpSolve.IMPROVE_DUALFEAS | LpSolve.IMPROVE_THETAGAP);
		this.engine.setBasiscrash(LpSolve.CRASH_NOTHING);
		this.engine.setSimplextype(LpSolve.SIMPLEX_DUAL_PRIMAL);
		this.engine.setVerbose(1);
		// this.engineType.setTimeout(timeOut);
		this.engine.setPresolve(LpSolve.PRESOLVE_ROWS | LpSolve.PRESOLVE_COLS | LpSolve.PRESOLVE_LINDEP,
				engine.getPresolveloops());
	}

	@Override
	public EngineType engineType() {
		return EngineType.LPSOLVE_DISPOSABLE;
	}

	@Override
	public boolean isFeasible() {
		boolean result = false;
		try {
			engine.setBreakAtFirst(true);
			if (engine.solve() == 0) {
				result = true;
			}
		} catch (LpSolveException e) {
			e.printStackTrace();
		}
		engine.deleteAndRemoveLp();
		return result;
	}

	@Override
	public Map<Integer, Double> solve() {
		Map<Integer, Double> result = new HashMap<Integer, Double>();
		try {
			if (engine.solve() == 0) {
				// get a pointer to the result and store it.
				double[] solution = engine.getPtrVariables();
				for (int variable = 0; variable < solution.length; variable++) {
					if (solution[variable] > 0) {
						result.put(variable + 1, solution[variable]);
					}
				}
			}
			return result;
		} catch (LpSolveException e) {
			e.printStackTrace();
			engine.deleteAndRemoveLp();
			return result;
		}
	}

	@Override
	public Pair<Map<Integer, Double>, Double> solveAndValue() {
		Map<Integer, Double> solution = new HashMap<Integer, Double>();
		double value;
		Pair<Map<Integer, Double>, Double> result = new Pair<Map<Integer, Double>, Double>(solution, null);
		try {
			if (engine.solve() == 0) {
				double[] rawSol = engine.getPtrVariables();
				for (int variable = 0; variable < rawSol.length; variable++) {
					if (rawSol[variable] > 0) {
						solution.put(variable + 1, rawSol[variable]);
					}
				}
				value = engine.getObjective();
				result = new Pair<Map<Integer, Double>, Double>(solution, value);
			}
		} catch (LpSolveException e) {
			e.printStackTrace();
			result = new Pair<Map<Integer, Double>, Double>(new HashMap<Integer, Double>(), null);
		}
		engine.deleteAndRemoveLp();
		return result;
	}

	@Override
	public Pair<double[], Double> solveAndValueNative() {
		double[] solution = new double[0];
		Double objectiveValue = null;
		try {
			if (engine.solve() == 0) {
				double[] raw = engine.getPtrVariables();
				solution = new double[raw.length + 1];
				System.arraycopy(raw, 0, solution, 1, raw.length);
				objectiveValue = engine.getObjective();
			}
			return new Pair<double[], Double>(solution, objectiveValue);
		} catch (LpSolveException e) {
			e.printStackTrace();
			engine.deleteAndRemoveLp();
			return new Pair<double[], Double>(solution, objectiveValue);
		}
	}

	@Override
	public double[] solveNative() {
		double[] solution = new double[0];
		try {
			if (engine.solve() == 0) {
				double[] raw = engine.getPtrVariables();
				solution = new double[raw.length + 1];
				System.arraycopy(raw, 0, solution, 1, raw.length);
			}
			return solution;
		} catch (LpSolveException e) {
			e.printStackTrace();
			engine.deleteAndRemoveLp();
			return solution;
		}
	}
}
